import datetime
from dataclasses import asdict

from valor import Annotation, Dataset, Datum, Filter, Label, Model
from valor.enums import TaskType


def test_empty_filter():
    f = asdict(Filter())
    assert f == {
        "dataset_names": None,
        "dataset_metadata": None,
        "model_names": None,
        "model_metadata": None,
        "datum_uids": None,
        "datum_metadata": None,
        "task_types": None,
        "annotation_metadata": None,
        "require_bounding_box": None,
        "bounding_box_area": None,
        "require_polygon": None,
        "polygon_area": None,
        "require_raster": None,
        "raster_area": None,
        "labels": None,
        "label_ids": None,
        "label_keys": None,
        "label_scores": None,
    }


def test_declarative_filtering():
    filters = [
        Dataset.name == "dataset1",
        Model.name == "model1",
        Datum.uid == "uid1",
        Label.key == "k1",
        Label.score > 0.5,
        Label.score < 0.75,
        Annotation.labels == [Label(key="k2", value="v2")],
        Annotation.task_type.in_(
            [TaskType.CLASSIFICATION, TaskType.OBJECT_DETECTION]
        ),
        # geometry filters
        Annotation.raster.is_none(),
        Annotation.polygon.is_none(),
        Annotation.bounding_box.is_not_none(),
        Annotation.bounding_box.area >= 1000,
        Annotation.bounding_box.area <= 5000,
        # metadata filters
        Dataset.metadata["arbitrary_numeric_key"] >= 10,
        Dataset.metadata["arbitrary_numeric_key"] < 20,
        Model.metadata["arbitrary_str_key"] == "arbitrary value",
        Datum.metadata["arbitrary_datetime_key"] >= datetime.timedelta(days=1),
        Datum.metadata["arbitrary_datetime_key"] <= datetime.timedelta(days=2),
        Annotation.metadata["myKey"] == "helloworld",
    ]

    f = asdict(Filter.create(filters))
    assert f == {
        "dataset_names": ["dataset1"],
        "dataset_metadata": {
            "arbitrary_numeric_key": [
                {"value": 10, "operator": ">="},
                {"value": 20, "operator": "<"},
            ]
        },
        "model_names": ["model1"],
        "model_metadata": {
            "arbitrary_str_key": [
                {"value": "arbitrary value", "operator": "=="}
            ]
        },
        "datum_uids": ["uid1"],
        "datum_metadata": {
            "arbitrary_datetime_key": [
                {"value": {"duration": 86400.0}, "operator": ">="},
                {"value": {"duration": 172800.0}, "operator": "<="},
            ]
        },
        "task_types": [
            TaskType.CLASSIFICATION.value,
            TaskType.OBJECT_DETECTION.value,
        ],
        "require_bounding_box": True,
        "bounding_box_area": [
            {"value": 1000, "operator": ">="},
            {"value": 5000, "operator": "<="},
        ],
        "annotation_metadata": {
            "mykey": [{"value": "helloworld", "operator": "=="}]
        },
        "require_polygon": False,
        "polygon_area": None,
        "require_raster": False,
        "raster_area": None,
        "labels": [{"k2": "v2"}],
        "label_ids": None,
        "label_keys": ["k1"],
        "label_scores": [
            {"value": 0.5, "operator": ">"},
            {"value": 0.75, "operator": "<"},
        ],
    }



import numpy as np
import pytest

from valor import Label


def test_label():
    # valid
    l1 = Label(key="test", value="value")

    # test validation
    with pytest.raises(TypeError):
        assert Label(key=123, value="123")  # type: ignore - testing
    with pytest.raises(TypeError):
        assert Label(key="123", value=123)  # type: ignore - testing

    # test member fn `tuple`
    assert l1.tuple() == ("test", "value", None)

    # test member fn `__eq__`
    l2 = Label(key="test", value="value")
    assert l1 == l2

    # test member fn `__ne__`
    l3 = Label(key="test", value="other")
    assert l1 != l3

    # test member fn `__hash__`
    assert l1.__hash__() == l2.__hash__()


def test_scored_label():
    l1 = Label(key="test", value="value")

    # valid
    s1 = Label(key="test", value="value", score=0.5)
    s2 = Label(key="test", value="value", score=0.5)
    s3 = Label(key="test", value="value", score=0.1)
    s4 = Label(key="test", value="other", score=0.5)
    s5 = Label(key="other", value="value", score=0.5)
    s6 = Label(key="test", value="value", score=np.float32(0.5))

    # test validation
    with pytest.raises(TypeError):
        assert Label(key="k", value="v", score="boo")  # type: ignore

    # test property `key`
    assert l1.key == "test"

    # test property `value`
    assert l1.value == "value"

    # test member fn `__eq__`
    assert (s1 == s2).get_value()  # type: ignore - resolved to 'Bool' as both sides are values
    assert (s1 == s6).get_value()  # type: ignore - resolved to 'Bool' as both sides are values
    assert not (s1 == s3).get_value()  # type: ignore - resolved to 'Bool' as both sides are values
    assert not (s1 == s4).get_value()  # type: ignore - resolved to 'Bool' as both sides are values
    assert not (s1 == s5).get_value()  # type: ignore - resolved to 'Bool' as both sides are values
    with pytest.raises(TypeError):
        assert s1 == 123
    with pytest.raises(TypeError):
        assert s1 == "123"

    # test member fn `__ne__`
    assert not (s1 != s2).get_value()  # type: ignore - resolved to 'Bool' as both sides are values
    assert (s1 != s3).get_value()  # type: ignore - resolved to 'Bool' as both sides are values
    assert (s1 != s4).get_value()  # type: ignore - resolved to 'Bool' as both sides are values
    assert (s1 != s5).get_value()  # type: ignore - resolved to 'Bool' as both sides are values
    with pytest.raises(TypeError):
        assert s1 != 123
    with pytest.raises(TypeError):
        assert s1 != "123"

    # test member fn `__hash__`
    assert s1.__hash__() == s2.__hash__()
    assert s1.__hash__() != s3.__hash__()
    assert s1.__hash__() != s4.__hash__()
    assert s1.__hash__() != s5.__hash__()



import datetime

import pytest

from valor import Annotation, Dataset, Filter, Label, Model
from valor.coretypes import _format_filter
from valor.schemas import Polygon
from valor.schemas.filters import Constraint


@pytest.fixture
def polygon() -> Polygon:
    coordinates = [
        [
            (125.2750725, 38.760525),
            (125.3902365, 38.775069),
            (125.5054005, 38.789613),
            (125.5051935, 38.71402425),
            (125.5049865, 38.6384355),
            (125.3902005, 38.6244225),
            (125.2754145, 38.6104095),
            (125.2752435, 38.68546725),
            (125.2750725, 38.760525),
        ]
    ]
    return Polygon(coordinates)


@pytest.fixture
def geojson(polygon: Polygon):
    return {"type": "Polygon", "coordinates": polygon.get_value()}


def test__format_filter(geojson, polygon):

    filter_object = Filter(
        dataset_names=["a", "b", "c"],
        model_names=["x", "y", "z"],
        label_scores=[Constraint(value=0.75, operator=">")],
        polygon_area=[
            Constraint(value=1000, operator=">"),
            Constraint(value=5000, operator="<"),
        ],
        raster_area=[
            Constraint(value=100, operator=">"),
            Constraint(value=500, operator="<"),
        ],
        dataset_metadata={
            "some_str": [Constraint(value="foobar", operator="==")],
            "some_float": [Constraint(value=0.123, operator=">=")],
            "some_datetime": [
                Constraint(
                    value={
                        "duration": datetime.timedelta(days=1).total_seconds()
                    },
                    operator=">",
                )
            ],
            "some_geospatial": [
                Constraint(
                    value=geojson,
                    operator="intersect",
                )
            ],
        },
    )

    filter_from_constraints = Filter.create(
        [
            Dataset.name.in_(["a", "b", "c"]),
            (Model.name == "x") | Model.name.in_(["y", "z"]),
            Label.score > 0.75,
            Annotation.polygon.area > 1000,
            Annotation.polygon.area < 5000,
            (Annotation.raster.area > 100) & (Annotation.raster.area < 500),
            Dataset.metadata["some_str"] == "foobar",
            Dataset.metadata["some_float"] >= 0.123,
            Dataset.metadata["some_datetime"] > datetime.timedelta(days=1),
            Dataset.metadata["some_geospatial"].intersects(polygon),
        ]
    )

    filter_from_dictionary = _format_filter(
        {
            "dataset_names": ["a", "b", "c"],
            "model_names": ["x", "y", "z"],
            "label_scores": [{"value": 0.75, "operator": ">"}],
            "polygon_area": [
                {"value": 1000, "operator": ">"},
                {"value": 5000, "operator": "<"},
            ],
            "raster_area": [
                {"value": 100, "operator": ">"},
                {"value": 500, "operator": "<"},
            ],
            "dataset_metadata": {
                "some_str": [{"value": "foobar", "operator": "=="}],
                "some_float": [{"value": 0.123, "operator": ">="}],
                "some_datetime": [
                    {
                        "value": {
                            "duration": datetime.timedelta(
                                days=1
                            ).total_seconds()
                        },
                        "operator": ">",
                    }
                ],
                "some_geospatial": [
                    {
                        "value": geojson,
                        "operator": "intersect",
                    }
                ],
            },
        }
    )

    assert filter_object == filter_from_constraints
    assert filter_object == filter_from_dictionary



import numpy as np

from valor.type_checks import is_float, is_numeric


def test_is_numeric():
    assert is_numeric(int(1))
    assert is_numeric(float(0.5))
    assert is_numeric(np.float32(0.5))
    assert not is_numeric(None)
    assert not is_numeric("hello world")


def test_is_float():
    assert is_float(float(0.5))
    assert is_float(np.float32(0.5))
    assert not is_float(int(1))
    assert not is_float(None)
    assert not is_float("hello world")
